
#
# Copyright 2001, 2002 by Fle3 Team and contributors
#
# This file is part of Fle3.
#
# Fle3 is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Fle3 is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Fle3; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""Contains class UserManager, which is the holder and factory object
for FLE users."""

__version__ = "$Revision: 1.8 $"[11:-2]

from time import time
import types, Errors
from urllib import quote_plus

import re
import string
import Globals
import OFS
from Globals import Persistent, Acquisition, PersistentMapping
from OFS.Image import Image
from AccessControl import ClassSecurityInfo

from TraversableWrapper import TraversableWrapper
from Cruft import Cruft
from UserDict import UserDict

from UserInfo import UserInfo
from Webtop import Webtop
from common import get_url, reload_dtml, add_dtml, new_reload_dtml, \
     make_action, get_roles, get_local_roles
from common import perm_view, perm_edit, perm_manage, perm_add_lo
from input_checks import is_valid_title, is_valid_id
from Products.ZWiki.ZWikiPage import ZWikiPage

# Instance of this class is FLE/fle_users and contains
# all UserInfo objects.
# This class contains factory methods for creating and removing users.
# User editing is in the UserInfo class.
class UserManager(
    TraversableWrapper,
    Cruft,
    Persistent,
    OFS.Folder.Folder,
    OFS.SimpleItem.Item,
    ):
    """FLE user manager."""

    meta_type = 'UserManager'
    dtml_files = (
        ('show_pending', 'Lists pending users',
         'ui/UserManager/show_pending'),
        ('invite_user_form', 'Invite User Form',
         'ui/UserManager/invite_user_form'),
        ('register_user_form', 'Register User Form',
         'ui/UserManager/register_user_form'),
        ('remove_user_form', 'Remove User Form',
         'ui/UserManager/remove_user_form'),
        ('remove_users_confirm', 'Remove Users Confirmation',
         'ui/UserManager/remove_users_confirm'),
        ('add_user_form_1_3', 'Add user form (1/3)',
         'ui/UserManager/add_user_form_1_3'),
        ('edit_user_form_1_3', 'Edit user form(1/3)',
         'ui/UserManager/edit_user_form_1_3'),

        ('fle_form_header', 'Standard Html Header for forms (UM)',
         'ui/UserManager/fle_form_header'),

        ('fle_html_header', 'Standard FLE Html Header (UM)',
         'ui/UserManager/fle_html_header'),

        # Taken from UserInfo.py
        ('show_user_info', 'user info page',
         'ui/UserInfo/show_user_info'),
        ('user_info', 'actual content',
         'ui/UserInfo/user_info'),

        ('edit_user_form', '',
         'ui/UserInfo/edit_user_form'),
        ('kasutajate_haldamise_vorm', 'Haldamise tarbeks',
         'ui/UserManager/kasutajate_haldamise_vorm'),
        ('kasutajaotsing', 'Administraatorile',
         'ui/UserManager/kasutajaotsing'),
        ('kasutajanimistu', 'Administraatorile',
         'ui/UserManager/kasutajanimistu'),
	('oigustePiiramine', 'Administraatorile',
	 'ui/UserManager/oigustePiiramine'),

        )

    security = ClassSecurityInfo()
    security.declareObjectPublic()
    security.declareProtected(perm_edit, 'edit_user_form')


##     def __bobo_traverse__(self,request,entry_name=None):
##         if entry_name in self.objectIds('UserInfo'):
##             return getattr(self.UserInfo,entry_name)
##         return getattr(self,entry_name)

    def index_html(self,REQUEST):
	" index_html to fle_users "
	return REQUEST.RESPONSE.redirect(self.fle_root().absolute_url())

    # No additional comments.
    def __init__(self, id, title):
        """Construct FLE User Manager."""
        self.id = id
        self.title = title

        self.pending_users = PersistentMapping()
        self.groups = []

        for tup in self.dtml_files:
            add_dtml(self, tup)

    security.declarePrivate('manage_afterAdd')
    def manage_afterAdd(self, item, container):
        """Set default permissions for roles."""
        from common import roles_admin, roles_staff, roles_user

        self.manage_permission(perm_manage, roles_admin, 0)
        self.manage_permission(perm_edit, roles_staff, 0)
        self.manage_permission(perm_view, roles_staff, 0)

        getattr(self, 'fle_form_header').manage_permission(
            perm_view,
            ['Authenticated', 'Anonymous',], 1)

    # Groups are stored in a list.
    security.declarePrivate('add_group')
    def add_group(self, name):
        """Add a new group."""
        if name not in self.groups:
            self.groups.append(name)

    # No additional comments.
    security.declarePrivate('remove_group')
    def remove_group(self, name):
        """Remove a group."""
        if name in self.groups:
            self.groups.remove(name)

    # No additional comments.
    security.declarePrivate('get_groups')
    def get_groups(self):
        """Get list of groups."""
        return self.groups[:]

    security.declarePublic('has_user_global_role')
    def has_user_global_role(self, uname, role):
        """Has given user given role."""
        return role in get_roles(self.parent(),uname)

    security.declarePublic('is_power_user')
    def is_power_user(self, uname):
        """Is user a power user?"""

        if not type(uname) is types.StringType:
            # Try to convert uname to string (it is a REQUEST.AUTHENTICATED_USER)
            uname = str(uname)
        if not self.acl_users.getUser(uname):
            return 0
        return getattr(self.fle_users,uname).has_any_role(('FLEAdmin', 'Manager', 'Staff'))

    security.declarePublic('lang_from_auth')
    def lang_from_auth(self, auth):
        """Get language for pending user that has given auth string."""
        for key in self.pending_users.keys():
            if self.pending_users[key][0] == auth:
                return self.pending_users[key][2]

        return 'en'

    # --> this method can be accessed by anyone.
    # --> Be careful to check that only valid invited
    #     users can use this (and only once!) --jmp
    security.declarePublic('add_invited_user')
    def add_invited_user(
        self,
        uname,
        password,
        first_name,
        last_name,
        email,
        auth,
        REQUEST):
        """Add invited user (and remove her from pending users).
        This method is called when an invited user applies the URL
        he/she received in the invitation email."""
        auth_strings = []

        for e in self.pending_users.values():
            auth_strings.append(e[0])

        if auth not in auth_strings:
            return 'Permission denied.' + auth + '-' + str(auth_strings)

        errors = []
        self.get_lang(('common', 'usermgmnt'), REQUEST)

        # Mandatory
        for x in ('uname', 'password'):
            if not is_valid_id(eval(x)):
                errors.append(REQUEST['L_' + x])

        for x in ('first_name', 'last_name'):
            if not is_valid_title(eval(x)):
                errors.append(REQUEST['L_' + x])

        # Optional
        if email and not is_valid_title(email):
            errors.append(REQUEST['L_email'])

        if len(errors) > 0:
            return self.message_dialog_error(
                self, REQUEST,
                title=REQUEST['L_invalid_input'],
                message=REQUEST['L_invalid_fields'] + ": '" + \
                "' , '".join(errors) + "'",
                action=apply(
                make_action,
                ['register_user_form'] +
                [(x, eval(x)) for x in
                 ('uname', 'first_name', 'last_name', 'email', 'auth')]))

        # Remove user from the pending users.
        for key in self.pending_users.keys():
            if self.pending_users[key][0] == auth:
                language = self.pending_users[key][2]
                course_ids = self.pending_users[key][4]
                self.remove_pending_user(key)
                break

        self.add_user_form_handler(
            uname,
            password,
            password,
            first_name,
            last_name,
            email,
            language = language,
            role = 'User',
            add = 'add',
            REQUEST = REQUEST)

        # Add user to course(s).
        for course_id in course_ids:
            course = self.courses.get_child(course_id)
            course.set_roles(uname, ('Student',))
            if course.has_group_folder():
                course.make_group_folder_proxies((self.get_user_info(uname),))

        if REQUEST:
            REQUEST.RESPONSE.redirect(self.state_href(
                REQUEST, 'fle_users/'+uname+'/edit_user_form'))

    security.declarePrivate('is_nickname_free')
    def is_nickname_free(self, nickname):
        """Checks that given nick name is not already take by somebody
        else. Note that case doesn't matter: if somebody is 'foo'
        nobody can take 'Foo'."""
        return nickname.lower() not in \
               [o.lower() for o in self.acl_users.getUserNames()]

    # Called from coursemanager's course_selection.dtml
    security.declarePublic('is_valid_uname')
    def is_valid_uname(self, uname):
        """Public valid uname checking. Checks if the given user name
        exists."""
        return uname in self.acl_users.getUserNames()

    security.declareProtected(perm_edit, 'add_user_form_handler')
    def add_user_form_handler(
        self,

        # actual user info data from forms
        uname,
        password1,
        password2,
        first_name,
        last_name,
        email = '',
        homepage = '',
        organization = '',
        language = '',
        role = 'User',

        previous = '', # submit buttons
        next = '',
        add = '',
        cancel = '',

        REQUEST = None):
        """Handles user creation in UserManager context. See UserInfo.edit_user_form_handler for user editing."""
        self.get_lang(('common','usermgmnt'),REQUEST)

        if cancel:
            if REQUEST:
                REQUEST.RESPONSE.redirect(
                    self.state_href(REQUEST, 'index_html'))
            return

        if not (uname and password1 and password2 and first_name and \
                last_name):
            return self.message_dialog_error(
                self, REQUEST,
                title=REQUEST['L_error'],
                message=REQUEST['L_required_field_missing'],
                action="add_user_form_1_3?uname=%s&first_name=%s&last_name=%s&email=%s&homepage=%s&organization=%s&language=%s&role=%s" % tuple(
                [quote_plus(o) for o in (uname, first_name, last_name, email,
                                         homepage, organization, language,
                                         role)])
                )

        if not self.is_nickname_free(uname):
            return self.message_dialog_error(
                self, REQUEST,
                title=REQUEST['L_error'],
                message=REQUEST['L_reserved_nickname'],
                action="add_user_form_1_3?uname=%s&first_name=%s&last_name=%s&email=%s&homepage=%s&organization=%s&language=%s&role=%s" % tuple(
                [quote_plus(o) for o in (uname, first_name, last_name, email,
                                         homepage, organization, language,
                                         role)])
                )
        if password1 != password2:
            return self.message_dialog_error(
                self, REQUEST,
                title=REQUEST['L_error'],
                message=REQUEST['L_password_mismatch'],
                action="add_user_form_1_3?uname=%s&first_name=%s&last_name=%s&email=%s&homepage=%s&organization=%s&language=%s&role=%s" % tuple(
                [quote_plus(o) for o in (uname, first_name, last_name, email,
                                         homepage, organization, language,
                                         role)])
                )
        password = password1

        errors = []
        self.get_lang(('common', 'usermgmnt'), REQUEST)

        if not is_valid_id(uname):
            errors.append(REQUEST['L_nickname'])

        for x in ('first_name', 'last_name'):
            if not is_valid_title(eval(x)):
                errors.append(REQUEST['L_' + x])

        for x in ('email', 'homepage', 'organization', 'language'):
            if eval(x) and not is_valid_title(eval(x)):
                errors.append(REQUEST['L_' + x])

        if len(errors) > 0:
            return self.message_dialog_error(
                self, REQUEST,
                title=REQUEST['L_invalid_input'],
                message=REQUEST['L_invalid_fields'] + ": '" + \
                "' , '".join(errors) + "'",
                action=apply(
                make_action,
                ['add_user_form_1_3'] +
                [(x, eval(x)) for x in
                 ('uname', 'first_name', 'last_name', 'email', 'homepage',
                  'organization', 'language', 'role')]))

        # This is ugly: we have to make some extra checks because:
        #  * Users with role 'Staff' can create only 'User' can 'Staff' users.
        #  * Users with role 'FLEAdmin' can create any kind of users.
        logged_user = self.get_current_user(REQUEST)
        if role == 'User':
            roles = ('User',)
        elif role == 'Staff':
            # This check is not really needed as this method
            # is already protected so that we know that logged
            # user is at least 'Staff', but let's be paranoid...
            if self.has_user_global_role(logged_user, 'Staff') or \
               self.has_user_global_role(logged_user, 'FLEAdmin'):
                roles = ('User', 'Staff')
            else:
                raise 'FLE Error', \
                      'Only Staff or FLEadmin can' + \
                      'create new users with role Staff'
        elif role == 'FLEAdmin':
            if self.has_user_global_role(logged_user, 'FLEAdmin'):
                roles = ('User', 'FLEAdmin')
            else:
                raise 'FLE Error', \
                      'Only FLEadmin can create new users with role FLEAdmin'
        else:
            raise 'FLE Error', 'Unknown role'

        uo = self.add_user(uname, password, roles)
        uo.set_first_name(first_name)
        uo.set_last_name(last_name)
        if email: uo.set_email(email)
        if homepage: uo.set_homepage(homepage)
        if language: uo.set_language(language)
        if organization: uo.set_organization(organization)

        if REQUEST:
            if add:
                REQUEST.RESPONSE.redirect(self.state_href(
                    REQUEST, 'index_html'))
            elif next:
                url = '%s/edit_user_form_2_3' % (uname)
                REQUEST.RESPONSE.redirect(self.state_href(REQUEST, url))
            else:
                raise 'FLE Error', ("Unknown thing happened.", REQUEST)

    # If user with given uname does not exist an exception is raised.
    security.declareProtected(perm_edit, 'freeze_user')
    def freeze_user(self, uname):
        """Freeze user."""
        user = self._getOb(uname)
        if user.is_frozen():
            raise 'FLE Error','User %s is already frozen!' % uname

        # Staff and fleadmins aren't frozen
        if user.has_any_role(('FLEAdmin','Staff')):
            raise 'FLE Error','User %s is Staff or FLEAdmin!' % uname
        user.freeze_me()

    # If user with given uname does not exist an exception is raised.
    security.declareProtected(perm_edit, 'unfreeze_user')
    def unfreeze_user(self, uname):
        """Unfreeze user."""
        acl_user_obj = self.acl_users.getUser(uname)
        user = self._getOb(uname)

        if not user.is_frozen():
            raise 'FLE Error','User %s is not frozen!' % uname
        user.unfreeze_me()
        
        
        
        
    security.declareProtected(perm_edit, 'pakuKasutajaleNimi')
    def pakuKasutajaleNimi(self, algnimi="stud"):
      """Proovitakse nimesid. Kui algne nimi sobib, 
        siis jetakse see, muul juhul asutakse numbreid lisama.
        kuni leitakse vaba"""
      if not self.kasKasutajaOlemas(algnimi):
         return algnimi
      nr=1
      while self.kasKasutajaOlemas(algnimi+str(nr)):
         nr=nr+1
      return algnimi+str(nr)
      
      
    security.declareProtected(perm_edit, 'kasKasutajaOlemas')     
    def kasKasutajaOlemas(self, nimi):
      "Vaadatakse lbi kuni leitakse vi jutakse juurikast krgemale"
      obj=self
      while hasattr(obj, 'acl_users'):
        if nimi in obj.acl_users.getUserNames():
          return 1
        try:
          obj=obj.aq_parent
        except:
          return 0
      return 0
      
    def leiaKasutajaKataloog(self, nimi):
      "Kui leitakse siis vljastatakse, muul juhul 0"
      obj=self
      while 1:
        if hasattr(obj, 'acl_users'):
          if nimi in obj.acl_users.getUserNames():
            return obj.acl_users
        try:
         obj=obj.aq_parent
        except:
         return 0
      
    def kasutajanimedeLoetelu(self):
      "leitavate nimede nimistu"
      obj=self
      loetelu=[]
      while 1:
        if hasattr(obj, 'acl_users'):
          for nimi in obj.acl_users.getUserNames():
            if nimi not in loetelu:
              loetelu.append(nimi)
        try:
         obj=obj.aq_parent
        except:
         break
      return loetelu
      
    security.declareProtected(perm_edit, 'lisaKasutaja')             
    def lisaKasutaja(self, nimi, parool):
      "Kasutaja lisamine"
      if self.kasKasutajaOlemas(nimi): return 0
      self.acl_users._doAddUser(nimi, parool, ['Member',], ())
      self.add_user_fle(nimi, ('Member',))
      return "Kasutaja lisatud"
      
    security.declareProtected(perm_edit, 'looParool')           
    def looParool(self):
      "Generaator"
      from random import random
      return str(int(random()*10000000))

    security.declareProtected(perm_edit, 'looKasutaja')
    def looKasutaja(self, kasutajanimi):
      "Nimi kohandatakse, luuakse parool, vljastatakse mlemad"
      nimi=self.pakuKasutajaleNimi(kasutajanimi)
      parool=self.looParool()
      if self.lisaKasutaja(nimi, parool):
        return (nimi, parool)
      return None
      
    security.declareProtected(perm_edit, 'kasutajateLisamiseVorm')          
    def kasutajateLisamiseVorm(self, REQUEST):
      "Tekstiala lisamiseks"
      self.get_lang(('ivacommon',), REQUEST)
      tulemus=self.hdr(self, REQUEST)+\
        " &gt; "+REQUEST['L_ic_user_adding']+" <br />"+\
        "<h3>"+REQUEST['L_ic_user_names']+"</h3>"+\
        "<form action='kasutajateLisamiseLeht'>"+\
        "<textarea name='kasutajanimed'></textarea><br />"+\
        "<input type='submit' value='"+REQUEST['L_ic_user_create']+"' />"+\
        "</form>"+\
        self.fle_html_footer(self, REQUEST)
      return tulemus
      
    security.declareProtected(perm_edit, 'kasutajateLisamiseLeht')
    def kasutajateLisamiseLeht(self, REQUEST, kasutajanimed):
      "Nimede jrgi lisamine. Parool genereeritakse, nimi kohandatakse"
      if not str(REQUEST.AUTHENTICATED_USER).lower()=='admin': return ":-("
      nimed=kasutajanimed.split()
      tulemus=" &gt; kasutajate lisamine <br />"
      for x in nimed:
         t=self.looKasutaja(x)
         if t is not None:
           tulemus=tulemus+x+": "+t[0]+", "+t[1]+"<br />"
         else:
           tulemus=tulemus+x+": probleem"+"<br />"
      return self.hdr(self, REQUEST)+tulemus+self.fle_html_footer(self, REQUEST)
      
    security.declareProtected(perm_edit, 'kasutajateLisamisFailiVorm')          
    def kasutajateLisamisFailiVorm(self, REQUEST):
      "Tekstiala lisamiseks"
      self.get_lang(('ivacommon',), REQUEST)
      tulemus=self.hdr(self, REQUEST)+\
        " &gt; <a href='kasutajate_haldamise_vorm'>"+REQUEST['L_ic_user_adding']+"</a> <br />"+\
        "<form action='kasutajaLehelt1'><table>"+\
        "<tr><td>"+REQUEST['L_ic_firstname']+"</td><td><input type='text' name='eesnimi' /></td></tr>"+\
        "<tr><td>"+REQUEST['L_ic_lastname']+"</td><td><input type='text' name='perenimi' /></td></tr>"+\
        "<tr><td>"+REQUEST['L_ic_username']+"</td><td><input type='text' name='kasutajanimi' /></td></tr></table>"+\
        "<input type='submit' value='"+REQUEST['L_ic_try']+"' /></form>"+\
        "<h3>"+REQUEST['L_ic_user_names']+"</h3>"+\
        "<form action='kasutajadFailist1' method='post' enctype='multipart/form-data'>"+\
        "<input type='file' name='fail' />"+\
        "<input type='submit' value='"+REQUEST['L_ic_register']+"' />"+\
        "</form>"+\
        self.fle_html_footer(self, REQUEST)
      return tulemus


    def kasutajaLehelt1(self, REQUEST, eesnimi, perenimi, kasutajanimi):
      "Kasutaja lisamine"
      nimeloetelu=self.kasutajanimedeLoetelu()
      nimeloetelu.sort()
      self.get_lang(('ivacommon',), REQUEST)
      eesnimi=eesnimi.strip()
      perenimi=perenimi.strip()
      kasutajanimi=kasutajanimi.strip()
      if len(eesnimi)==0 or len(perenimi)==0 or len(kasutajanimi)==0:
        return REQUEST['L_ic_missingdata']
      tulemus=self.hdr(self, REQUEST)+\
        "<table><form action='kasutajadFailist2'>"
      tulemus=tulemus+self.looIsikuRida(REQUEST, eesnimi, perenimi, kasutajanimi, nimeloetelu)
      tulemus=tulemus+"</table>"+self.erialadeKombo(REQUEST)+\
        "<input type='submit' value='"+REQUEST['L_ic_forward']+"' /></form>"+self.fle_html_footer(self, REQUEST)
      return tulemus      
      
    


    def kasutajadFailist1(self, REQUEST, fail):
      "CSV-failist lugemine, kasutajanimede kontroll"
      read=fail.readlines()
      kasutajanimed=[];
      #nimeloetelu=[x.get_uname() for x in self.fle_root().fle_users.get_users()]
      nimeloetelu=self.kasutajanimedeLoetelu()
      nimeloetelu.sort()
      self.get_lang(('ivacommon',), REQUEST)
      tulemus=self.hdr(self, REQUEST)
      tulemus=tulemus+"<form action='kasutajadFailist2' method='post'><table>"
      try:
       for rida in read:
        isik=rida.split(",") 
        enimi=unicode(isik[0].strip(),'iso-8859-1').encode('utf-8')
        pnimi=unicode(isik[1].strip(),'iso-8859-1').encode('utf-8')
        knimi=unicode(isik[2].strip(),'iso-8859-1').encode('utf-8')
        korduv=0
        if self.kasKasutajaOlemas(knimi): 
          korduv=1
        if knimi in kasutajanimed: korduv=1
        tulemus=tulemus+"\n<tr><td><input type='hidden' name='enimi' value='"+enimi+"' />"+enimi+\
                         "</td><td><input type='hidden' name='pnimi' value='"+pnimi+"' />"+pnimi+"</td><td>"
        if korduv:
          nr=0
          mitu=3
          while ((nimeloetelu[nr]<=knimi) and (nr<len(nimeloetelu))): 
            nr=nr+1
          alg=nr-mitu
          if alg<0:alg=0
          while alg<nr:
            tulemus=tulemus+nimeloetelu[alg]+"<br />"          
            alg=alg+1
          tulemus=tulemus+"<input type='text' name='knimi' value='"+knimi+"' /><br />"
          if nimeloetelu[nr]==knimi:
            alg=alg+1
          while alg<=nr+mitu and alg<len(nimeloetelu):
            tulemus=tulemus+nimeloetelu[alg]+"<br />"          
            alg=alg+1
        else:
          tulemus=tulemus+"<input type='hidden' name='knimi' value='"+knimi+"' />"+knimi
          kasutajanimed.append(knimi)
        tulemus=tulemus+"</td></tr>"
      except:
        return self.hdr(self, REQUEST)+"Probleem failist lugemisel"+self.fle_html_footer(self, REQUEST)
      tulemus=tulemus+"</table>"+self.erialadeKombo(REQUEST)+"<input type='submit' value='"+REQUEST['L_ic_forward']+"' /></form>"+self.fle_html_footer(self,REQUEST)
      return tulemus
      
    def looIsikuRida(self, REQUEST, enimi, pnimi,knimi, nimeloetelu):
        "ks rida tabelisse"
	import urllib
        from urllib import quote
        korduv=0
        if self.kasKasutajaOlemas(knimi): 
          korduv=1
        tulemus="\n<tr><td><input type='hidden' name='enimi' value='"+urllib.quote(enimi)+"'>"+enimi+\
                         "</td><td><input type='hidden' name='pnimi' value='"+pnimi+"'>"+pnimi+"</td><td>"
        if korduv:
          nr=0
          mitu=3
          while ((nimeloetelu[nr]<knimi) and (nr<len(nimeloetelu))): 
            nr=nr+1
          alg=nr-mitu
          if alg<0:alg=0
          while alg<nr+mitu and alg<len(nimeloetelu):
            tulemus=tulemus+nimeloetelu[alg]+"<br />"          
            alg=alg+1
          tulemus=tulemus+"<input type='text' name='knimi' value='"+knimi+"' /><br />"
        else:
          tulemus=tulemus+"<input type='hidden' name='knimi' value='"+knimi+"' />"+knimi
        tulemus=tulemus+"</td></tr>"
        return tulemus
      
    def kasutajadFailist2(self, REQUEST, enimi, pnimi, knimi, eriala='-1'):
      "Lisamine ise"
      paroolid=[]
      e=self.kysiErialad(REQUEST);
      self.get_lang(('ivacommon',), REQUEST)
      tulemus=self.hdr(self, REQUEST)
      tulemus=tulemus+"<table>"
      if type(enimi)==types.StringType:
        enimi=(enimi,)
        pnimi=(pnimi,)
        knimi=(knimi,)
      korduv=0
      for nimi in knimi:
        if self.kasKasutajaOlemas(nimi):
          return "Nimi "+nimi+" kordub."
      for i in range(len(knimi)):
        parool=self.looParool()
        paroolid.append(parool)
        self.lisaKasutaja(knimi[i], parool)
        getattr(self, knimi[i]).set_first_name(enimi[i])
        getattr(self, knimi[i]).set_last_name(pnimi[i])
        if int(eriala)>=0:
          getattr(self, knimi[i]).set_organization(e[int(eriala)])          
        tulemus=tulemus+"<tr><td>"+enimi[i]+"</td><td>"+pnimi[i]+"</td><td>"+knimi[i]+"</td><td>"+paroolid[i]+"</td></tr>\n"
      tulemus=tulemus+"</table>"+self.fle_html_footer(self, REQUEST)
      return tulemus      
        
    
    def kasKasutajanimiSobib(self, REQUEST, nimi):
      "nime sobivuse kontroll"
      if len(nimi)<2: return 0
      
      return 1
      

    def kysiErialad(self, REQUEST):
      "Erialade loetelu, kasutajainfos organization"
      if hasattr(self, 'erialad'):
        self.erialad.sort()
        return self.erialad
      else:
        return []
        
    def erialadeKombo(self, REQUEST, kastinimi='eriala',vaikimisiID='-1',opt=''):
      "Kombo opt == full siis koht edit_user_info"
      e=self.kysiErialad(REQUEST)
      kasutaja=self.get_user_info(self.get_current_user(REQUEST))
      if len(e)>0:
       tulemus="<select "
       if hasattr(kasutaja, 'perm_org'):
		if kasutaja.perm_org == 1 and opt == 'full':
			tulemus += 'disabled="disabled"'
       if opt == 'full':
          tulemus += " name='"+kastinimi+"' class=\"width240\"><option value=' '></option>"
       else:
          tulemus += " name='"+kastinimi+"' class=\"width240\"><option value='-1'></option>"
       for i in range(len(e)):
        #tulemus=tulemus+"<option value='"+str(i)+"' "
	if opt == 'full':
		tulemus += "<option value='"+e[i]+"' "
		if kasutaja.get_organization() == e[i]:
			tulemus=tulemus+" selected=\"selected\""
	else:
		tulemus += "<option value='"+str(i)+"' "
        if str(i)==vaikimisiID:
          tulemus=tulemus+" selected=\"selected\""
        tulemus=tulemus+">"+e[i]+"</option>\n"
       tulemus=tulemus+"</select>"
       return tulemus
      return ""

    def erialadeTabel(self, REQUEST):
      "Tabel koos linnukestega"
      tulemus="<table>"
      e=self.kysiErialad(REQUEST)
      for i in range(len(e)):
        tulemus=tulemus+"<tr><td><input type='checkbox' name='eriala' value='"+str(i)+"' /></td><td>"+e[i]+"</td></tr>\n"
      tulemus=tulemus+"</table>"
      return tulemus
      
    def erialadeHalduseVorm(self, REQUEST):
      "Vorm"
      self.get_lang(('ivacommon',), REQUEST)
      tulemus=self.hdr(self, REQUEST)
      tulemus=tulemus+"<p></p><h2>"+REQUEST['L_ic_adding_affil']+"</h2>"
      tulemus=tulemus+"<form action='erialaLisamiseLeht'>"+\
        "<input type='text' name='erialaNimi' /><input type='submit' value='OK' /></form><br />"
      tulemus=tulemus+"<p></p><h3>"+REQUEST['L_ic_import_affil']+"</h3>"+\
        "<form action='erialadeFailistLisamiseLeht' method='post' enctype='multipart/form-data'>"+\
        "<input type='file' name='erialadeLoetelu' /><input type='submit' value='"+REQUEST['L_ic_upload']+"' />"+"</form>"+\
        "<form action='erialadeNimekirjaHalduseLeht'>"+self.erialadeTabel(REQUEST)
      if len(self.kysiErialad(REQUEST))>0:
        tulemus=tulemus+"<input type='submit' value='"+REQUEST['L_ic_delete']+"' />"
      tulemus=tulemus+"</form>"+self.fle_html_footer(self, REQUEST)
      return tulemus
                 
      
    def erialaLisamiseLeht(self, REQUEST, erialaNimi):
      "Leht"
      e=self.kysiErialad(REQUEST)
      if erialaNimi not in e:
        e.append(erialaNimi)
      self.erialad=e
      REQUEST.RESPONSE.redirect('erialadeHalduseVorm')

    def erialadeFailistLisamiseLeht(self, REQUEST, erialadeLoetelu):
      "Failist"
      e=self.kysiErialad(REQUEST)
      loetelu=erialadeLoetelu.readlines()
      for x in loetelu:
        if x not in e:
          e.append(x)
      self.erialad=e
      REQUEST.RESPONSE.redirect('erialadeHalduseVorm')
            
    def erialadeNimekirjaHalduseLeht(self, REQUEST, eriala):
      "Kustutus"
      e=self.kysiErialad(REQUEST)
      if type(eriala)==None:eriala=()
      if type(eriala)==types.StringType:eriala=(eriala,)
      i=len(eriala)-1
      while i>=0:
        del e[i]
        i=i-1
      self.erialad=e
      REQUEST.RESPONSE.redirect('erialadeHalduseVorm')
      
        


    security.declarePrivate('add_user')
    # uname: User account name
    #
    # password: The password
    #
    # roles: List of (FLE-wide) roles the user should have
    #
    # domains: List of domains from which the user is allowed to login.
    def add_user(self, uname, password, roles, domains=()):
        """Add user to UserManager."""

        # If the user has no roles and no password, we'll just skip.
        if not password and not roles:
            return
        if uname in self.acl_users.getUserNames():
            raise 'FLE Error', "Trying to add user that already exists in acl_users! (%s)" % (uname)
        try:
            self.acl_users._doAddUser(uname, password, [], list(domains))
        except:
            import sys
            if sys.exc_type=='NotImplemented':
                raise Errors.FleError("The user folder is implemented partially and does not support adding users.")
            raise
        try:
            user = self.add_user_fle(uname,roles)
        except:
            # In case of exception, get rid of user in acl_users.
            try:
                self.acl_users._doDelUsers((uname,))
            except KeyError:
                pass
            except:
                import sys
                if sys.exc_type=='NotImplemented':
                    raise Errors.FleError("The user folder is implemented partially and does not support removing users.")
                raise
        return user

    def add_user_fle(self, uname, roles):
        user = UserInfo(uname)
        user.perm_org = self.fle_root().def_not_aff
        user.perm_pass = self.fle_root().def_not_change
        user.perm_must_change = self.fle_root().def_passwd
        #user.id=uname
        self._setObject(uname, user)
        # A clever loop to get to the actual acl_user folder which contains
        # the authenticated user (because it might not be the closest one!)
        obj=self
        while 1:
            acl_user_obj = obj.acl_users.getUser(uname)
            if acl_user_obj:
                acl_user_obj=acl_user_obj.__of__(obj.acl_users)
                break
            obj=obj.aq_parent
        # Get the user object from acl_users, and set it as the
        # owner of the newly created UserInfo object.
        user.changeOwnership(acl_user_obj)

        # Set the userthingy as a owner (local role) to the
        # newly created UserInfo object.
        user.manage_delLocalRoles(user.get_valid_userids())
        user.manage_setLocalRoles(uname, ('Owner',))
        # Set the user's FLE-global roles to the FLE root object.
        self.parent().manage_setLocalRoles(uname,roles)
        return user

    def _create_a_random_string(self):
        """Create a random string (used to authenticate invited users)."""
        from common import random, a_char
        import sha

        secret = ''
        m = sha.new()
        for i in range(5):
            secret = secret + str(random())
            m.update(secret)
            hash = m.digest()
            secret = ''
        for i in hash:
            secret = secret + a_char(ord(i))
        return secret


    # This uses FLE/MailHost object
    # Invited users are stored in self.pending_users which
    # is a dictionary containing secret key, invitation
    # message and language: {'email':(secret, message, language, URL1), ...}
    # Invitation message, language and URL1 are stored for possible
    # re-invitation.
    #
    # Parameters:
    #
    # emails: list of e-mail addresses
    #
    # message: message to be sent via email
    security.declareProtected(perm_edit, 'invite_user')
    def invite_user(self, REQUEST, emails, message, language, course_ids=[]):
        """Invite user by email."""
        import urllib

        self.get_lang(('common','usermgmnt'),REQUEST)
        # This is for calling method from test code
        URL1 = REQUEST.URL1

        if type(course_ids) == types.StringType:
            course_ids = (course_ids,)

        # Figure out who is sending this email.
        user_name = str(REQUEST.AUTHENTICATED_USER)
        send_address = self.get_user_info(user_name).get_email()
        if not send_address:
            # If the user wanting to invite people has no email address,
            # then she must set one, before she can send invitation messages.
            return self.message_dialog_error(
                self, REQUEST,
                title=REQUEST['L_no_email'],
                message=REQUEST['L_no_email_explanation'],
                action = apply(
                make_action,
                ['invite_user_form'] +
                [(x, eval(x)) for x in ('emails', 'message', 'language')] +
                [('course_ids', str(course_ids))]
                ))

        errors = []
        self.get_lang(('common', 'usermgmnt'), REQUEST)
        if not emails: # FIXME: input_checks: something more fancy?
            return self.message_dialog_error(
                self, REQUEST,
                title=REQUEST['L_invalid_input'],
                message=REQUEST['L_invalid_fields'] + ": '" + \
                REQUEST['L_email_addresses'] +"'",
                action = apply(
                make_action,
                ['invite_user_form'] +
                [(x, eval(x)) for x in ('emails', 'message', 'language')] +
                [('course_ids', str(course_ids))]
                ))

        # Check if user has hacked input.
        uinfo = self.fle_users.get_user_info(user_name)
        course_ids_of_user = [x.get_id() for x in uinfo.user_courses()]
        if not uinfo.has_role('FLEAdmin'):
            for course_id in course_ids:
                if course_id not in course_ids_of_user:
                    raise 'FLE Error', 'Hey you! You are not on that course!'

        # FIXME: input_checks: should message be checked?

        emails = string.replace(emails, ",", " ")
        emails = string.split(emails)

        for email in emails:

            # Create a random string (used to authenticate invited users)
            secret = self._create_a_random_string()

            try:
                self.__send_invitation(str(email),send_address,message,secret,language,URL1,course_ids)
            except AttributeError:
                return self.message_dialog_error(self, REQUEST,
                    title=REQUEST['L_error'],
                    message=REQUEST['L_could_not_send_invitation'],
                    action="index_html")

            # invitation message is needed for re-invitation
            self.pending_users[email] = (secret, message, language, URL1,
                                         course_ids)
            self._p_changed = 1

        return self.message_dialog(
            self, REQUEST,
            title=REQUEST['L_invitations_sent'],
            message=REQUEST['L_invitations_sent_to'] % emails,
            action='index_html')

    # An exception is raised if email is invalid.
    # FIXME: re-invitation needs invitation message to be stored
    # This uses FLE/MailHost object
    security.declareProtected(perm_edit, 're_invite_user')
    def re_invite_user(self, email, send_address):
        """Re-invite user by email."""

        secret, message, language, URL1, course_ids = self.pending_users[email]
        self.__send_invitation(send_address,email,message,secret,language,URL1,course_ids)

    def __send_invitation(self,to_address,from_address,message,secret,language,baseurl,course_ids):

        # encode secret authentication key for
        # transfering it in an url
        #
        import urllib
        secret_encoded = urllib.quote(secret)
        mailhost = self.MailHost
        to_encoded = urllib.quote(to_address)

        url = '%s/register_user?email=%s&auth=%s' % \
              (baseurl, to_encoded, secret_encoded)

        inv_lang_dict = {}
        self.get_lang_given(('common', 'usermgmnt'),
                            inv_lang_dict, language)

        msg = "Content-Type: text/plain; charset=UTF-8\n\n"+\
              inv_lang_dict['L_invitation_header']+'\n'+\
              message + '\n' + \
              inv_lang_dict['L_invitation_footer'] + url
        subject = inv_lang_dict['L_invitation_subject']

        mailhost.send(msg,to_address,from_address,subject,encode="quoted-printable")

    # This calls register_user_form.
    security.declarePublic('register_user')
    def register_user(self, email, auth, REQUEST):
        """Register a pending user (when invited user registers himself)."""
        import urllib

        self.get_lang(('common','usermgmnt'),REQUEST)

        if not email in self.pending_users.keys():
            return self.message_dialog_error(
                self, REQUEST,
                title=REQUEST['L_registration_failed'],
                message=REQUEST['L_invitation_key_used'],
                action="")

        # auth string must be decoded
        if self.pending_users[email][0] != urllib.unquote(auth):
            return self.message_dialog_error(
                self, REQUEST,
                title=REQUEST['L_registration_failed'],
                message=REQUEST['L_authorization_failed'],
                action="")

        REQUEST['email']=email
        REQUEST['auth']=auth
        return self.register_user_form(
            self, REQUEST)

    # FIXME: How to handle all the objects that the user owns and that have
    # links to user's information (like user's name or photo)?
    security.declareProtected(perm_manage, 'remove_user')
    def remove_user(self, uname):
        """Remove user."""
        if not uname in self.objectIds():
            raise 'FLE Error', 'Given user does not exist.'

        try:
            acl_users = self.acl_users
            acl_users._doDelUsers((uname,))
            self._delObject(uname)
        except:
            import sys
            if sys.exc_type=='NotImplemented':
                raise Errors.FleError("The user folder is implemented partially and does not support removing users.")
            raise
            

    security.declareProtected(perm_manage, 'kasutajaOigused')
    def kasutajaOigused(self, REQUEST, nimed='',perm_org='', perm_pass='', perm_must_change=''):
	" muudab kasutaja oiguseid "
	kasutajanimed = string.split(nimed,',')
	if type(perm_org)==types.StringType:
		perm_org=(perm_org,)
	if type(perm_pass)==types.StringType:
		perm_pass=(perm_pass,)
	if type(perm_must_change)==types.StringType:
		perm_must_change=(perm_must_change,)

	# Vaatame kas kasutajale lubatakse ogranisatsiooni vahetamist voi mitte

	for kas_kasutaja in kasutajanimed:
		if kas_kasutaja != '':
			lipuke = 0
			viga = 0
			for abc in perm_org:
				if abc != '':
					kasutaja = self.get_user_info(abc)
					if kas_kasutaja == str(kasutaja.get_uname()):
						lipuke = 1
						kasutaja.perm_org = 1
				else:
					viga = 1
			if lipuke==0:
				kasutaja = self.get_user_info(kas_kasutaja)
				kasutaja.perm_org = 0

	# Vaatame kas kasutajale lubatakse parooli vahetamist voi mitte

        for kas_kasutaja in kasutajanimed:
                if kas_kasutaja != '':
                        lipuke = 0
                        viga = 0
                        for abc in perm_pass:
                                if abc != '':
                                        kasutaja = self.get_user_info(abc)
                                        if kas_kasutaja == str(kasutaja.get_uname()):
                                                lipuke = 1
                                                kasutaja.perm_pass = 1
                                else:
                                        viga = 1
                        if lipuke==0:
                                kasutaja = self.get_user_info(kas_kasutaja)
                                kasutaja.perm_pass = 0

	# Vaatame kas kasutajale sunnitakse parooli vahetamine

        for kas_kasutaja in kasutajanimed:
                if kas_kasutaja != '':
                        lipuke = 0
                        viga = 0
                        for abc in perm_must_change:
                                if abc != '':
                                        kasutaja = self.get_user_info(abc)
                                        if kas_kasutaja == str(kasutaja.get_uname()):
                                                lipuke = 1
                                                kasutaja.perm_must_change = 1
                                else:
                                        viga = 1
                        if lipuke==0:
                                kasutaja = self.get_user_info(kas_kasutaja)
                                kasutaja.perm_must_change = 0


	return
	#REQUEST.RESPONSE.redirect('kasutajaotsing')

    security.declareProtected(perm_manage, 'kasutajateKustutus')
    def kasutajateKustutus(self, REQUEST, kasutajanimed, delete = '', permission = ''):
      "Kasutajate kustutamine"
      self.get_lang(('ivacommon',), REQUEST)
      tulemus=self.hdr(self, REQUEST)+"<br />"
      if type(kasutajanimed)==types.StringType:
      	kasutajanimed=(kasutajanimed,)
      if delete:
      	viga=0
      	for kasutajanimi in kasutajanimed:
      	   tulemus="<br />"+tulemus+kasutajanimi
      	   try:
      	    kasutaja=self.get_user_info(kasutajanimi)
      	    for kursus in kasutaja.user_courses():
      	      kursus.remove_person(kasutajanimi)
      	    self.remove_user(kasutajanimi)
      	    tulemus=tulemus+"+"
      	   except:
      	    tulemus=tulemus+"?"
      	    viga=1
      	tulemus+tulemus+self.fle_html_footer(self, REQUEST)
      	if viga:
      	  return tulemus
      elif permission:
	nimed = ''
	for kasutajanimi in kasutajanimed:
		nimed += kasutajanimi
		nimed += ','
      		#kasutaja=self.get_user_info(kasutajanimi) 
		#if hasattr(kasutaja, 'permissions'):
		#	if kasutaja.permissions:
		#		kasutaja.permissions = 0
		#	else:
		#		kasutaja.permissions = 1
		#else:
		#	kasutaja.permissions = 1
#	REQUEST.RESPONSE.redirect('oigustePiiramine')
#	return self.oigustePiiramine(nimed=nimed)
	REQUEST.RESPONSE.redirect('oigustePiiramine?nimed='+nimed)
      else:
	raise 'FLE error', 'unknown button in kasutajaotsing'
      if delete: 
        REQUEST.RESPONSE.redirect('kasutajaotsing?vormist=-1')

    # Country list lives in this method.
    security.declarePublic('get_countries')
    def get_countries(self):
        """Return list of all(?) countries in the world (in English)"""
        from countries import country_list
        return country_list

    security.declarePublic('get_user_info')
    def get_user_info(self, uname):
        """Return a the uname UserInfo object."""
        from types import StringType
        if not type(uname) is StringType:        
            raise 'FLE Error', 'Argument uname is not string type! (and this is the wrong place to convert it).'

        # Get user info object from fle_users folder
        try:
            return self.get_child(uname)
        except AttributeError:
            raise Errors.FleError('No such FLE user: %s' % (uname))
        except:
            return None

    security.declareProtected(perm_manage, 'reload_dtml')
    def reload_dtml(self, REQUEST=None):
        """Reload dtml files from the file system."""
        reload_dtml(self, self.dtml_files)

        # This page needs to be viewable without authentication.
        getattr(self,'register_user_form').manage_permission(
            perm_view,
            ['Authenticated', 'Anonymous',], 1)

        if REQUEST:
            self.get_lang(('common','usermgmnt'),REQUEST)
            return self.message_dialog(
                self, REQUEST,
                title=REQUEST['L_dtml_reloaded'],
                message=REQUEST['L_dtml_files_reloaded'],
                action='index_html')

    security.declareProtected(perm_manage, 'remove_users_confirm_handler')
    def remove_users_confirm_handler(
        self,
        REQUEST,
        user_string = '',
        remove = '',  # form submit buttons
        cancel = '',  #
        ):
        """Handle input from the remove_users_confirm.dtml"""
        if remove:
            self.get_lang(('common','usermgmnt'),REQUEST)
            success = []
            failure = []
            users = string.split(user_string)

            for user in users:
                try:
                    self.remove_user(user)
                    success.append(user)
                except:
                    failure.append(user)

            message=REQUEST['L_removed_users'] % success
            if failure:
                message += '<br>'+REQUEST['L_not_removed_users'] % failure

            return self.message_dialog(
                self, REQUEST,
                title=REQUEST['L_users_removed'],
                message=message,
                action='index_html')

        elif cancel:
            REQUEST.RESPONSE.redirect(self.state_href(REQUEST, 'index_html'))
            return
        else:
            raise 'FLE Error', 'Unknwon button' # Should never happen

    # No additional comments.
    security.declareProtected(perm_edit, 'form_handler')
    def form_handler(
        self,
        REQUEST,
        users = '',
        freeze = '',   # submit buttons
        unfreeze = '', #
        remove = '',   #
        ):
        """Handle input from the fle_users/index_html form."""
        if type(users) is types.StringType:
            users = [users,]

        if freeze:
            for user in users:
                self.freeze_user(user)
            REQUEST.RESPONSE.redirect(self.state_href(REQUEST, 'index_html'))
            return
        elif unfreeze:
            for user in users:
                self.unfreeze_user(user)
            REQUEST.RESPONSE.redirect(self.state_href(REQUEST, 'index_html'))
            return
        elif remove:
            user_string = users[0]
            for user in users[1:]:
                user_string += "%20" + user
            REQUEST.RESPONSE.redirect(
                self.state_href(
                REQUEST,
                'remove_users_confirm?user_string=%s' % user_string))
            return
        else:
            # This code should never be reached.
            raise 'FLE Error', 'Unknown button'

    # No additional comments.
    security.declareProtected(perm_edit, 'remove_pending_user')
    def remove_pending_user(self, email):
        """Remove pending user."""
        try:
            del self.pending_users[email]
        except KeyError:
            pass
        self._p_changed = 1

    # No additional comments.
    security.declareProtected(perm_view, 'get_pending_users')
    def get_pending_users(self):
        """Get emails of pending users."""
        return self.pending_users.keys()

    security.declarePublic('get_users')
    def get_users(self):
        """Return a list of UserInfo objects."""
        return self.get_children('UserInfo')

    def get_users_sorted_by_uname(self):
        """Return a list of UserInfo objects sorted by uname"""

        def uname_comp(x,y):
            a = x.get_uname()
            b = y.get_uname()
            if a < b:     return -1
            elif  a == b: return  0
            else:         return  1

        retval = self.get_children('UserInfo')
        retval.sort(uname_comp)
        return retval

        
#    def get_users_sorted_by(self, tunnus="uname"):
#       "sorteeritud kasutajad"
#
#       class sorteerija:        
#        def kysi_tunnus(k):
#         if t=="uname": return k.get_uname()
#         if t=="email": return k.get_email()
#         if t=="first_name": return k.get_first_name()
#         if t=="last_name": return k.get_last_name()
#        self.t=tunnus 
#       
#        def k_comp(x, y):
#         a=self.kysi_tunnus(x, self.t)
#         b=self.kysi_tunnus(y, self.t)
#         if a<b:    return -1
#         elif a==b: return 0
#         else:      return 1
#         
#       retval = self.get_children('UserInfo')
#       retval.sort(sorteerija.k_comp)
#       return retval
         

    security.declareProtected(perm_edit, 'make_temp_user')
    def make_temp_user(self, uname):
        """Make temporary user from existing user."""
        return self.fle_root().temp_objects.add_object(
            self.get_child(uname))

    # FIXME: input_checks
    # No additional comments.
    security.declareProtected(perm_edit, 'pending_users_form_handler')
    def pending_users_form_handler(
        self,
        REQUEST,
        users = '',
        remove = '',    # submit buttons
        re_invite = '', #
        cancel = '',    #
        ):
        """Handle input from the fle_users/show_pending.dtml form."""
        self.get_lang(('common','usermgmnt'),REQUEST)
        if cancel:
            # go back to the user management index page
            REQUEST.RESPONSE.redirect(self.state_href(
                REQUEST, 'index_html'))
            return

        elif re_invite:
            user_name = str(REQUEST.AUTHENTICATED_USER)
            send_address = self.get_user_info(user_name).get_email()
            if not send_address:
                self.get_lang(('common', 'usermgmnt'), REQUEST)

                # FIXME: selected users are not passed back!!!
                return self.message_dialog_error(
                    self, REQUEST,
                    title=REQUEST['L_no_email'],
                    message=REQUEST['L_no_email_explanation'],
                    action='show_pending')


            # FIXME: check that invitation succeeds
            # Make sure that we use a list of strings!
            if type(users) is types.StringType:
                users = [users,]
            count = 0
            for user in users:
                try:
                    self.re_invite_user(user, send_address)
                    count += 1
                except:
                    pass

            return self.message_dialog(
                self, REQUEST,
                title=REQUEST['L_re_invitations_sent'],
                message=REQUEST['L_re_invitations_sent_to'] % str(count),
                action='show_pending')

        elif remove:
            success = []
            failure = []

            # Make sure that we use a list of strings!
            if type(users) is types.StringType:
                users = [users,]

            for user in users:
                try:
                    self.remove_pending_user(user)
                    success.append(user)
                except:
                    failure.append(user)

            message = REQUEST['L_pending_removed_users'] % success
            if failure:
                message += '<br>'+ \
                           REQUEST['L_pending_not_removed_users'] % failure

            return self.message_dialog(
                self, REQUEST,
                title=REQUEST['L_pending_users_removed'],
                message=message,
                action='show_pending')

        else:
            # This code should never be reached.
            raise 'FLE Error', 'Unknown button'


    security.declareProtected(perm_manage, 'cleanup_webtops')
    def cleanup_webtops(self):
        """Sets webtop items' owners to be the webtop owners."""
        for user in self.get_users():
            print "Cleaning for user %s..." % user.get_uname()
            self.set_owner_of_webtop(user.webtop,user.get_uname())

    def set_owner_of_webtop(self,folder,uname):
        for item in folder.objectValues(('WebtopFolder','WebtopFile','WebtopLink','WebtopMemo')):
            print "Cleaning %s" % item.get_id()
            item.set_author(uname)
            if item.meta_type=='WebtopFolder':
                self.set_owner_of_webtop(item,uname)

    def export_users(self, REQUEST,failinimi='user_export.zip',kursusega='',palju='all'):
	""" expordi kasutajad """
	from ImportExportIMS import User_export
	import tempfile, os
	abc = User_export()
	if palju == 'all':
#		for user in self.filtreeriKasutajad(REQUEST, self.fle_root().fle_users.get_users()):
		for user in self.fle_root().fle_users.get_users():
			abc.kirjutaKasutaja(user)
	else:
		kursus=getattr(self.fle_root().courses, kursusega)
		for user in eval(palju):
			abc.kirjutaKasutaja(user,kursus)
		abc.kirjutaKursusGrupina(kursus)
		abc.kirjutaMembership(kursus)
		for krupp in kursus.sisegrupid.objectValues('GroupFolder'):
			abc.kirjutaSisegrupid(kursus,krupp)
			abc.kirjutaMembership(krupp,'krupp')
	abc.lisaMembership()
	abc.pakiFaili(str(kursusega)+"/",failinimi)
	# kui tuleme course.py-st et exportida kursus koos kasutajatega
	if kursusega:
		return
	file = open(failinimi,"rb")
	export_data=file.read()
	file.close()
	os.remove(failinimi)
	REQUEST.RESPONSE.setHeader('Content-disposition','attachment; filename=user_export.zip')
	REQUEST.RESPONSE.setHeader('content-type','application/zip')
	return export_data

    def kysiKasutaja(self, knimi=""):
      "Test"
      return self.acl_users.getUser(knimi).getDomains()
     
    def kasutajaGsm(self, gsmnr='0'):
      "Kasutajanime leidmine gsm numbri alusel"
      kasutajad=[]
      for user in self.get_users():
        if user.get_gsm()== gsmnr:
          kasutajad.append( user.get_uname())
      if len(kasutajad)>0:
        return kasutajad
      return 'none'

    def emailChanges(self,REQUEST):
	""" email changes to user """
	# 24h tunni piire 86400ms == 24h
	from common import FakeRequest
	import time
	ajavahe = 86000.0
	hetkeaeg = time.time()
	if hasattr(self,'last_notification'):
		if hetkeaeg - self.last_notification < ajavahe:
			#print "too soon to do anything"+str(hetkeaeg - self.last_notification)
			return "too soon to do anything"+str(hetkeaeg - self.last_notification)
	else:
		self.last_notification = time.time()
	self.last_notification = time.time()
	kasutajad = self.get_users()
	kirjad_ootamisel = []
	self.get_lang(('common','ivacommon',),REQUEST)
	for kasutaja in kasutajad:
		if not kasutaja.get_email().strip():
			continue
		f_req = FakeRequest(kasutaja.get_uname())
		kursused = kasutaja.user_courses()
		s_kiri = ""
		for kursus in kursused:
			kiri = ""
			try:
		         if hasattr(kasutaja,'email_teadm'):
			   if kasutaja.email_teadm:
			    if kursus.kasTeadmuspajasKontekste():
				jump =  kursus.tootoaMuutused['teadmuspaja']-kursus.tootoaKylastused['teadmuspaja'][kasutaja.get_uname()] 
				jump2 = hetkeaeg - kursus.tootoaMuutused['teadmuspaja']
				if jump > 0 and jump < ajavahe and jump2 <ajavahe:
					kiri += "- "+REQUEST['L_ic_knowledge_building_link']+" \n"
			except:
				if hetkeaeg - kursus.tootoaMuutused['teadmuspaja'] < ajavahe:
					kiri += "- "+REQUEST['L_ic_knowledge_building_link']+" \n"
			try:
			 if hasattr(kasutaja,'email_meedia'):
			   if kasutaja.email_meedia:
			    if kursus.kasMeediapajasProjekte():
				jump =  kursus.tootoaMuutused['meediapaja']-kursus.tootoaKylastused['meediapaja'][kasutaja.get_uname()]
				jump2 = hetkeaeg - kursus.tootoaMuutused['meediapaja']
				if jump > 0 and jump < ajavahe and jump2 <ajavahe:
					kiri += "- "+REQUEST['L_ic_jamming_link']+" \n"
			except:
				if hetkeaeg - kursus.tootoaMuutused['meediapaja'] < ajavahe:
					kiri += "- "+REQUEST['L_ic_jamming_link']+" \n"
			if hasattr(kasutaja,'email_raamat'):
			  if kasutaja.email_raamat:
			    jump = kursus.gf.viimaneMuutus()-kursus.gf.viimaneKylastus(f_req)
			    jump2 = hetkeaeg - kursus.gf.viimaneMuutus()
			    if jump > 0 and jump < ajavahe and jump2 < ajavahe:
				    kiri += "- "+REQUEST['L_bookshelf']+" \n"
			    elif not kursus.gf.viimaneKylastus(f_req) and jump2 < ajavahe:
				    kiri += "- "+REQUEST['L_bookshelf']+" \n"
			    try:
				wt = getattr(kasutaja.webtop, 'c'+kursus.get_id())
				jump = wt.viimaneMuutus()-wt.viimaneKylastus(f_req)
				jump2 = hetkeaeg - wt.viimaneMuutus()
				if jump > 0 and jump < ajavahe and jump2 < ajavahe:
					kiri += "- "+REQUEST['L_webtop']+" \n"
				elif not wt.viimaneKylastus(f_req) and jump2 < ajavahe:
					kiri += "- "+REQUEST['L_webtop']+" \n"
			    except:
				pass
			if kiri:
				#s_kiri += "kursusel "+kursus.get_name()+" on muutunud:\n"
                                s_kiri += REQUEST['L_ic_course_changed'] % kursus.get_name()
				s_kiri += ":\n"+kiri+"\n"
		if not s_kiri:
			continue
		kirja_algus = "\n\n"+REQUEST['L_ic_email_1']+" \n" 
		kirja_algus += time.strftime(REQUEST['L_timestamp_format'], time.localtime(hetkeaeg-86400))+" - "
		kirja_algus += time.strftime(REQUEST['L_timestamp_format'], time.localtime(hetkeaeg))
		kirja_algus += "\n"+REQUEST['L_ic_email_2']+":\n"
		s_kiri = "\n" + str(kirja_algus) +"\n"+ s_kiri
#		s_kiri = "\nKiri aadressile "+ kasutaja.get_email().strip()+s_kiri+"\n"
#		print s_kiri
#		self.saadaEmail(str(kasutaja.get_email().strip()),'Muutused IVA kursustel',str(s_kiri))
		kirjad_ootamisel.append((str(kasutaja.get_email().strip()),str(s_kiri)))
		# see algusesse
                subjekt = REQUEST['L_ic_e_subject']
	for kiri in kirjad_ootamisel:
		self.saadaEmail(kiri[0],subjekt,kiri[1],kiri[0])
	return "ok"

    def fix_pictures(self):
	""" fixes user pictures """
        kasutajad = self.get_users()
	for kasutaja in kasutajad:
		pilt = kasutaja.get_photo()
		kasutaja.set_photo(pilt)

	return "ok"

Globals.InitializeClass(UserManager)
# EOF
